;6502 8 bit PRNG
;http://retro.hansotten.nl/6502-sbc/lee-davison-web-site/some-code-bits/#prng
; by Lee Davison
; returns pseudo random 8 bit number in A. Affects A, X and Y.
; (r_seed) is the byte from which the number is generated and MUST be
; initialised to a non zero value or this function will always return
; zero. Also r_seed must be in RAM, you can see why......
 
rand_8
    LDA r_seed      ; get seed
    AND #$B8        ; mask non feedback bits
                ; for maximal length run with 8 bits we need
                ; taps at b7, b5, b4 and b3
    LDX #$05        ; bit count (shift top 5 bits)
    LDY #$00        ; clear feedback count
F_loop
    ASL A       ; shift bit into carry
    BCC bit_clr     ; branch if bit = 0
 
    INY         ; increment feedback count (b0 is XOR all the   
                ; shifted bits from A)
bit_clr
    DEX         ; decrement count
    BNE F_loop      ; loop if not all done
 
no_clr
    TYA         ; copy feedback count
    LSR A       ; bit 0 into Cb
    LDA r_seed      ; get seed back
    ROL A       ; rotate carry into byte
    STA r_seed      ; save number as next seed
    RTS         ; done
 
r_seed
    .byte   1       ; prng seed byte (must not be zero)